@@ -1,5 +1,5 @@
 /* Implicit rule searching for GNU Make.
-Copyright (C) 1988, 89, 90, 91, 92, 93, 94 Free Software Foundation, Inc.
+Copyright (C) 1988,89,90,91,92,93,94,97 Free Software Foundation, Inc.
 This file is part of GNU Make.
 
 GNU Make is free software; you can redistribute it and/or modify
@@ -14,7 +14,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Make; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+Boston, MA 02111-1307, USA.  */
 
 #include "make.h"
 #include "rule.h"
@@ -35,7 +36,7 @@ try_implicit_rule (file, depth)
      struct file *file;
      unsigned int depth;
 {
-  DEBUGPR ("Looking for an implicit rule for `%s'.\n");
+  DEBUGPR (_("Looking for an implicit rule for `%s'.\n"));
 
   /* The order of these searches was previously reversed.  My logic now is
      that since the non-archive search uses more information in the target
@@ -50,7 +51,7 @@ try_implicit_rule (file, depth)
      archive member name to search for implicit rules.  */
   if (ar_name (file->name))
     {
-      DEBUGPR ("Looking for archive-member implicit rule for `%s'.\n");
+      DEBUGPR (_("Looking for archive-member implicit rule for `%s'.\n"));
       if (pattern_search (file, 1, depth, 0))
 	return 1;
     }
@@ -111,14 +112,14 @@ pattern_search (file, archive, depth, recursions)
   /* This buffer records all the dependencies actually found for a rule.  */
   char **found_files = (char **) alloca (max_pattern_deps * sizeof (char *));
   /* Number of dep names now in FOUND_FILES.  */
-  unsigned int deps_found;
+  unsigned int deps_found = 0;
 
   /* Names of possible dependencies are constructed in this buffer.  */
   register char *depname = (char *) alloca (namelen + max_pattern_dep_length);
 
   /* The start and length of the stem of FILENAME for the current rule.  */
-  register char *stem;
-  register unsigned int stemlen;
+  register char *stem = 0;
+  register unsigned int stemlen = 0;
 
   /* Buffer in which we store all the rules that are possibly applicable.  */
   struct rule **tryrules
@@ -148,11 +149,11 @@ pattern_search (file, archive, depth, recursions)
      that is not just `%'.  */
   int specific_rule_matched = 0;
 
-  register unsigned int i;
+  register unsigned int i = 0;  /* uninit checks OK */
   register struct rule *rule;
   register struct dep *dep;
 
-  char *p;
+  char *p, *vp;
 
 #ifndef	NO_ARCHIVES
   if (archive || ar_name (filename))
@@ -167,6 +168,17 @@ pattern_search (file, archive, depth, recursions)
       lastslash = rindex (filename, ']');
 #else
       lastslash = rindex (filename, '/');
+#if defined(__MSDOS__) || defined(WINDOWS32)
+      /* Handle backslashes (possibly mixed with forward slashes)
+	 and the case of "d:file".  */
+      {
+	char *bslash = rindex (filename, '\\');
+	if (lastslash == 0 || bslash > lastslash)
+	  lastslash = bslash;
+	if (lastslash == 0 && filename[0] && filename[1] == ':')
+	  lastslash = filename + 1;
+      }
+#endif
 #endif
       if (lastslash != 0 && lastslash[1] == '\0')
 	lastslash = 0;
@@ -187,7 +199,7 @@ pattern_search (file, archive, depth, recursions)
 	 don't use it here.  */
       if (rule->in_use)
 	{
-	  DEBUGP2 ("Avoiding implicit rule recursion.%s%s\n", "", "");
+	  DEBUGP2 (_("Avoiding implicit rule recursion.%s%s\n"), "", "");
 	  continue;
 	}
 
@@ -235,11 +247,11 @@ pattern_search (file, archive, depth, recursions)
 	  if (check_lastslash)
 	    {
 	      if (stem > (lastslash + 1)
-		  && strncmp (target, lastslash + 1, stem - lastslash - 1))
+		  && !strneq (target, lastslash + 1, stem - lastslash - 1))
 		continue;
 	    }
 	  else if (stem > filename
-		   && strncmp (target, filename, stem - filename))
+		   && !strneq (target, filename, stem - filename))
 	    continue;
 
 	  /* Check that the rule pattern matches the text after the stem.
@@ -323,7 +335,7 @@ pattern_search (file, archive, depth, recursions)
 	      stemlen -= (lastslash - filename) + 1;
 	    }
 
-	  DEBUGP2 ("Trying pattern rule with stem `%.*s'.\n",
+	  DEBUGP2 (_("Trying pattern rule with stem `%.*s'.\n"),
 		   (int) stemlen, stem);
 
 	  /* Try each dependency; see if it "exists".  */
@@ -331,6 +343,8 @@ pattern_search (file, archive, depth, recursions)
 	  deps_found = 0;
 	  for (dep = rule->deps; dep != 0; dep = dep->next)
 	    {
+              struct file *fp;
+
 	      /* If the dependency name has a %, substitute the stem.  */
 	      p = index (dep_name (dep), '%');
 	      if (p != 0)
@@ -362,16 +376,16 @@ pattern_search (file, archive, depth, recursions)
 		     "impossible", then the rule fails and don't
 		     bother trying it on the second pass either
 		     since we know that will fail too.  */
-		  DEBUGP2 ("Rejecting impossible %s dependency `%s'.\n",
-			   p == depname ? "implicit" : "rule", p);
+		  DEBUGP2 (_("Rejecting impossible %s prerequisite `%s'.\n"),
+			   p == depname ? _("implicit") : _("rule"), p);
 		  tryrules[i] = 0;
 		  break;
 		}
 
 	      intermediate_files[deps_found] = 0;
 
-	      DEBUGP2 ("Trying %s dependency `%s'.\n",
-		       p == depname ? "implicit" : "rule", p);
+	      DEBUGP2 (_("Trying %s prerequisite `%s'.\n"),
+		       p == depname ? _("implicit") : _("rule"), p);
 
 	      /* The DEP->changed flag says that this dependency resides in a
 		 nonexistent directory.  So we normally can skip looking for
@@ -379,19 +393,25 @@ pattern_search (file, archive, depth, recursions)
 		 dependency file we are actually looking for is in a different
 		 directory (the one gotten by prepending FILENAME's directory),
 		 so it might actually exist.  */
+              /* If we find a file but the intermediate flag is set, then it
+                 was put here by a .INTERMEDIATE: rule so ignore it.  */
 
 	      if ((!dep->changed || check_lastslash)
-		  && (lookup_file (p) != 0 || file_exists_p (p)))
+		  && (((fp = lookup_file (p)) != 0 && !fp->intermediate)
+                      || file_exists_p (p)))
 		{
-		  found_files[deps_found++] = savestring (p, strlen (p));
+		  found_files[deps_found++] = xstrdup (p);
 		  continue;
 		}
 	      /* This code, given FILENAME = "lib/foo.o", dependency name
 		 "lib/foo.c", and VPATH=src, searches for "src/lib/foo.c".  */
-	      if (vpath_search (&p, (time_t *) 0))
+	      vp = p;
+	      if (vpath_search (&vp, (FILE_TIMESTAMP *) 0))
 		{
-		  DEBUGP2 ("Found dependency as `%s'.%s\n", p, "");
-		  found_files[deps_found++] = p;
+		  DEBUGP2 (_("Found prerequisite `%s' as VPATH `%s'\n"),
+                           p, vp);
+		  strcpy (vp, p);
+		  found_files[deps_found++] = vp;
 		  continue;
 		}
 
@@ -405,15 +425,15 @@ pattern_search (file, archive, depth, recursions)
 		    intermediate_file
 		      = (struct file *) alloca (sizeof (struct file));
 
-		  DEBUGP2 ("Looking for a rule with %s file `%s'.\n",
-			   "intermediate", p);
+		  DEBUGP2 (_("Looking for a rule with %s file `%s'.\n"),
+			   _("intermediate"), p);
 
 		  bzero ((char *) intermediate_file, sizeof (struct file));
 		  intermediate_file->name = p;
 		  if (pattern_search (intermediate_file, 0, depth + 1,
 				      recursions + 1))
 		    {
-		      p = savestring (p, strlen (p));
+		      p = xstrdup (p);
 		      intermediate_patterns[deps_found]
 			= intermediate_file->name;
 		      intermediate_file->name = p;
@@ -422,7 +442,7 @@ pattern_search (file, archive, depth, recursions)
 		      /* Allocate an extra copy to go in FOUND_FILES,
 			 because every elt of FOUND_FILES is consumed
 			 or freed later.  */
-		      found_files[deps_found] = savestring (p, strlen (p));
+		      found_files[deps_found] = xstrdup (p);
 		      ++deps_found;
 		      continue;
 		    }
@@ -505,6 +525,7 @@ pattern_search (file, archive, depth, recursions)
 	  f->deps = imf->deps;
 	  f->cmds = imf->cmds;
 	  f->stem = imf->stem;
+          f->also_make = imf->also_make;
 	  imf = lookup_file (intermediate_patterns[deps_found]);
 	  if (imf != 0 && imf->precious)
 	    f->precious = 1;
@@ -513,6 +534,9 @@ pattern_search (file, archive, depth, recursions)
 	  for (dep = f->deps; dep != 0; dep = dep->next)
 	    {
 	      dep->file = enter_file (dep->name);
+              /* enter_file uses dep->name _if_ we created a new file.  */
+              if (dep->name != dep->file->name)
+                free (dep->name);
 	      dep->name = 0;
 	      dep->file->tried_implicit |= dep->changed;
 	    }
@@ -554,7 +578,7 @@ pattern_search (file, archive, depth, recursions)
       dep->next = file->deps;
       file->deps = dep;
     }
- 
+
   if (!checked_lastslash[foundrule])
     /* Always allocate new storage, since STEM might be
        on the stack for an intermediate file.  */
@@ -572,10 +596,9 @@ pattern_search (file, archive, depth, recursions)
 
   file->cmds = rule->cmds;
 
-  /* Put the targets other than the one that
-     matched into FILE's `also_make' member.  */
+  /* If this rule builds other targets, too, put the others into FILE's
+     `also_make' member.  */
 
-  /* If there was only one target, there is nothing to do.  */
   if (rule->targets[1] != 0)
     for (i = 0; rule->targets[i] != 0; ++i)
       if (i != matches[foundrule])
